#version 110
#extension GL_EXT_gpu_shader4 : enable

uniform sampler2D DiffuseSampler;
uniform sampler2D DataSampler;

varying vec2 texCoord;
varying vec2 oneTexel;

uniform vec2 InSize;

#define Px 1.0/InSize.x
#define Py 1.0/InSize.y
#define SAME texture2D(DataSampler, texCoord)
#define DATA texture2D(DataSampler, vec2(0.0, 0.0)).r
#define FADE texture2D(DataSampler, vec2(Px * 2.0, 0.0)).r
#define OLDI texture2D(DataSampler, vec2(Px * 3.0, 0.0)).r
#define TIME toInt(texture2D(DataSampler, vec2(Px * 4.0, 0.0)).rgb)

int toInt(vec3 a) {
    return int(a.r * 255.) + (int(a.g * 255.) +  int(a.b * 255.) * 256) * 256;
}

vec3 toVec3(int a) {
    return vec3(float(a % 256) / 255., float((a % 65536) / 256) / 255., float((a % 16777216) / 65536) / 255.);
}

void get() {
    vec4 InTexel = texture2D(DiffuseSampler, vec2(0.5, 0.5));
    if(InTexel.a != 0.0 && InTexel.a != 1.0 && InTexel.r + InTexel.g + InTexel.b == 0.0 && InTexel.a < 0.5) gl_FragColor = vec4(InTexel.a, InTexel.a, InTexel.a, 1.0);
    else gl_FragColor = SAME;
}

void fade() {
    vec4 InTexel = texture2D(DiffuseSampler, vec2(0.5, 0.5));
    if(InTexel.a != 0.0 && InTexel.a != 1.0 && InTexel.r + InTexel.g + InTexel.b == 0.0 && InTexel.a < 0.5 && InTexel.a != texture2D(DataSampler, vec2(0.0, 0.0)).r) gl_FragColor = vec4(0.0, 0.0, 0.0, 1.0);
    else {
        gl_FragColor = SAME;
        gl_FragColor.r += 0.005;
    }
}

void previous() {
    vec4 InTexel = texture2D(DiffuseSampler, vec2(0.5, 0.5));
    vec4 Data = texture2D(DataSampler, vec2(0.0, 0.0));
    if(InTexel.a != 0.0 && InTexel.a != 1.0 && InTexel.r + InTexel.g + InTexel.b == 0.0 && InTexel.a < 0.5 && InTexel.a != Data.r) gl_FragColor = Data;
    else gl_FragColor = SAME;
}

void time() {
    vec4 time = SAME;
    int t = toInt(time.rgb);
    t++;
    time.rgb = toVec3(t);
    gl_FragColor = time;
}

void effectTime() {
    vec4 InTexel = texture2D(DiffuseSampler, vec2(0.5, 0.5));
    if(InTexel.a != 0.0 && InTexel.a != 1.0 && InTexel.r + InTexel.g + InTexel.b == 0.0 && InTexel.a < 0.5 && InTexel.a != texture2D(DataSampler, vec2(0.0, 0.0)).r) gl_FragColor = vec4(texture2D(DataSampler, vec2(Px * 4.0, 0.0)).rgb, 1.0);
    else gl_FragColor = SAME;
}

void sunEvent() {
    vec4 time = SAME;
    int t = toInt(time.rgb);
    t--; if(t < 0) t = 0;
    time.rgb = toVec3(t);
    vec4 InTexel = texture2D(DiffuseSampler, vec2(0.5, 0.5));
    if(InTexel.a != 0.0 && InTexel.a != 1.0 && InTexel.r + InTexel.g + InTexel.b == 0.0 && InTexel.a > 0.5) {
        int a = int(InTexel.a * 255.0);
        if(a == 128) time.rgb = toVec3(375);
    }
    gl_FragColor = time;
}

void main() {
    if(texCoord.y < Py) {
        if(texCoord.x < Px) get(); //Particle input
        else if(texCoord.x < 2.0 * Px) fade();
        else if(texCoord.x < 3.0 * Px) previous();
        else if(texCoord.x < 4.0 * Px) time();
        else if(texCoord.x < 5.0 * Px) effectTime();
        else if(texCoord.x < 6.0 * Px) sunEvent();
    }
}