#version 110

uniform sampler2D DiffuseSampler;
uniform sampler2D DataSampler;
uniform sampler2D BGSampler;
uniform sampler2D OVSampler;
uniform sampler2D IconSampler;

varying vec2 texCoord;
varying vec2 oneTexel;

#define TIME toInt(texture2D(DataSampler, vec2(0.0, 0.0)).rgb)

#define ICON_W 0.009375
#define ICON_H 0.050926

int toInt(vec3 a) {
    return int(a.r * 255.) + (int(a.g * 255.) +  int(a.b * 255.) * 256) * 256;
}

vec3 hsv2rgb(vec3 c) {
    vec4 K = vec4(1.0, 2.0 / 3.0, 1.0 / 3.0, 3.0);
    vec3 p = abs(fract(c.xxx + K.xyz) * 6.0 - K.www);
    return c.z * mix(K.xxx, clamp(p - K.xxx, 0.0, 1.0), c.y);
}

vec3 rgb2hsv(vec3 c)
{
    vec4 K = vec4(0.0, -1.0 / 3.0, 2.0 / 3.0, -1.0);
    vec4 p = mix(vec4(c.bg, K.wz), vec4(c.gb, K.xy), step(c.b, c.g));
    vec4 q = mix(vec4(p.xyw, c.r), vec4(c.r, p.yzx), step(p.x, c.r));

    float d = q.x - min(q.w, q.y);
    float e = 1.0e-10;
    return vec3(abs(q.z + (q.w - q.y) / (6.0 * d + e)), d / (q.x + e), q.x);
}

void main() {

    vec4 result = texture2D(BGSampler, vec2(texCoord.x, 1.0-texCoord.y));
    int t = TIME;
    
    if(texture2D(DiffuseSampler, vec2(0.0,0.0)) == vec4(1.0,1.0,1.0,1.0)) {
        vec4 over = texture2D(OVSampler, vec2(texCoord.x, 1.0-texCoord.y));
        if(over.a != 0.0) result = over;
        float time = float(t) / 75.0 / 3.0;
        if(result.r != 0.0 && result.g < 0.2 && result.g < 0.2 && result.b < 0.2)
            result += vec4(hsv2rgb(vec3(time, 1.0, pow(abs(texCoord.x - 0.5), 2.0))), 1.0);
        if(texCoord.y > 0.61 && length(result.rgb) > 0.5) {
            vec3 temp = rgb2hsv(result.rgb);
            result = vec4(hsv2rgb(vec3(time - texCoord.x - texCoord.y, temp.y, temp.z)), 1.0);
        }
    } else {
        float y = texCoord.y + sin(float(t) / 20.0) * 0.01;
        if(texCoord.x > ICON_W * 94.0 && texCoord.x < ICON_W * 95.0 && y > ICON_H * 10.0 && y < ICON_H * 11.0)
            result += texture2D(IconSampler, vec2(texCoord.x * 106.6666, (ICON_H - y) * 19.6666));
    }    

    gl_FragColor = result;
    
}
